﻿using System;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Polly;
using Polly.Contrib.Simmy;
using Polly.Contrib.Simmy.Behavior;
using Polly.Contrib.Simmy.Fault;
using Polly.Registry;
using Polly.Wrap;
using SimmyBreakingYourOwnCode.Service;

namespace SimmyBreakingYourOwnCode
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            IPolicyRegistry<string> registry = services.AddPolicyRegistry();

            HttpResponseMessage faultHttpResponseMessage = new HttpResponseMessage(HttpStatusCode.InternalServerError)
            {
                Content = new StringContent("Simmy swapped the Ok for an Internal Server Error")
            };
            AsyncInjectOutcomePolicy<HttpResponseMessage> faultPolicy = MonkeyPolicy.InjectFaultAsync(
                faultHttpResponseMessage,
                injectionRate: .5,
                enabled: () => true 
            );
            

            // Add the HttpClientFactory
            services.AddHttpClient("OpenBreweryDb", client =>
            {
                client.BaseAddress = new Uri("https://api.openbrewerydb.org/breweries/");
                client.DefaultRequestHeaders.Add("Accept", "application/json");
            });//.AddPolicyHandler(faultAndRetryWrap);

            services.AddSingleton<RandomValueGenerator>();
            services.AddMvc().SetCompatibilityVersion(CompatibilityVersion.Version_2_2);
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IHostingEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseMvc();
        }
    }
}
